****************************************************
* Klasse fr die Umwandlung der Texte in Base64- und HEX-Werte und umgekehrt
*
* Mindestvoraussetzungen: Windows XP bzw. Windows Server 2003
*
****************************************************
*
* Author: Andreas Engler
* Erstellt: 07.12.2010
* Copyright: Andreas Engler
* Version: 1.0
****************************************************

#include "Dll.ch"
#INCLUDE "cryptclass.CH"

CLASS CryptClass

        EXPORTED:
        METHOD Init


        METHOD StringToBase64        //Ein String in Base64-String umwandeln
        METHOD Base64ToString        //Ein Base64-String in String umwandeln
        METHOD Base64HeaderToString //Ein Base64-String mit Header in String umwandeln
        METHOD StringToBase64Header //Ein String in Base64-String mit Header umwandeln
        METHOD StringToHex                //Ein String in HEX-String umwandeln
        METHOD HexToString                //Ein Hex-String in String umwandeln
        METHOD StringToHexAddr                //Ein String in HEX-String inkl. Adressen umwandeln
        METHOD HexAddrToString                //Ein Hex-String inkl. Adressen in String umwandeln
        METHOD StringToHexAsciiAddr                //Ein String in HEX-String inkl. Adressen und ASCII-Code umwandeln
        METHOD HexAsciiAddrToString                //Ein Hex-String inkl. Adressen und ASCII-Code in String umwandeln

        //Allgemeine Methoden
        METHOD BynaryToString        //Ein String in einen String des angegebenen Typs umwandeln.
        METHOD StringToBinary        //einen String des angegebenen Typs in einen normalen String umwandeln

ENDCLASS


METHOD Init()


RETURN SELF

METHOD Base64ToString( cString )
        /* Beschreibung : Ein Base64-String in String umwandeln
                Parameter    : cString - Base64-String, dass in String umgewandelt werden soll
        */
        LOCAL cReturn := ""        //Rckgabewert

        cReturn := ::StringToBinary( cString, CRYPT_STRING_BASE64 )

RETURN cReturn

METHOD StringToBase64( cString )
        /* Beschreibung : Ein String in Base64-String umwandeln
                Parameter    : cString - Text, dass in Base64 kodierten Text umgewandelt werden soll
                Rckgabe     : cReturn - Base64-String
        */
        LOCAL cReturn := ""        //Rckgabewert

        cReturn := ::BynaryToString( cString, CRYPT_STRING_BASE64 )

RETURN cReturn

METHOD Base64HeaderToString( cString )
        /* Beschreibung : Ein Base64-String mit Header in String umwandeln
                Parameter    : cString - Base64-String, dass in String umgewandelt werden soll
        */
        LOCAL cReturn := ""        //Rckgabewert

        cReturn := ::StringToBinary( cString, CRYPT_STRING_BASE64HEADER )

RETURN cReturn

METHOD StringToBase64Header( cString )
        /* Beschreibung : Ein String in Base64-String mit Header umwandeln
                Parameter    : cString - Text, dass in Base64 kodierten Text umgewandelt werden soll
                Rckgabe     : cReturn - Base64-String
        */
        LOCAL cReturn := ""        //Rckgabewert

        cReturn := ::BynaryToString( cString, CRYPT_STRING_BASE64HEADER )

RETURN cReturn

METHOD StringToHex( cString )
        /* Beschreibung : Ein String in HEX-String umwandeln
                Parameter    : cString - Text, dass in HEX umgewandelt werden soll
                Rckgabe     : cReturn - HEX-String
        */
        LOCAL cReturn := ""        //Rckgabewert

        cReturn := ::BynaryToString( cString, CRYPT_STRING_HEX )

RETURN cReturn


METHOD HexToString( cString )
        /* Beschreibung : Ein Hex-String in String umwandeln
                Parameter    : cString - Hex-String, dass in String umgewandelt werden soll
                Rckgabe     : cReturn - String
        */
        LOCAL cReturn := ""         //Rckgabewert

        cReturn := ::StringToBinary( cString, CRYPT_STRING_HEX )

RETURN cReturn


METHOD StringToHexAddr( cString )
        /* Beschreibung : Ein String in HEX-String inkl. Adressen umwandeln
                Parameter    : cString - Text, dass in HEX umgewandelt werden soll
                Rckgabe     : cReturn - HEX-String inkl. Adressen
        */
        LOCAL cReturn := ""        //Rckgabewert

        cReturn := ::BynaryToString( cString, CRYPT_STRING_HEXADDR )

RETURN cReturn


METHOD HexAddrToString( cString )
        /* Beschreibung : Ein Hex-String inkl. Adressen in String umwandeln
                Parameter    : cString - Hex-String, dass in String umgewandelt werden soll
                Rckgabe     : cReturn - String
        */
        LOCAL cReturn := ""        //Rckgabewert

        cReturn := ::StringToBinary( cString, CRYPT_STRING_HEXADDR )

RETURN cReturn


METHOD StringToHexAsciiAddr( cString )
        /* Beschreibung : Ein String in HEX-String inkl. Adressen und ASCII-Code umwandeln
                Parameter    : cString - Text, dass in HEX umgewandelt werden soll
        */
        LOCAL cReturn := ""         //Rckgabewert

        cReturn := ::BynaryToString( cString, CRYPT_STRING_HEXASCIIADDR )

RETURN cReturn


METHOD HexAsciiAddrToString( cString )
        /* Beschreibung : Ein Hex-String inkl. Adressen und ASCII-Code in String umwandeln
                Parameter    : cString - Hex-String, dass in String umgewandelt werden soll
                Rckgabe     : cReturn - String
        */
        LOCAL cReturn := ""         //Rckgabewert

        cReturn := ::StringToBinary( cString, CRYPT_STRING_HEXASCIIADDR )

RETURN cReturn


METHOD BynaryToString( cString , dwCryptTyp )
        /* Beschreibung : Ein String in einen String des angegebenen Typs umwandeln.
                                                        Diese Methode ist eine Auslagerung des allgemenen in allen Methoden wiederholten Quellcodes
                Parameter    : cString - String
                                                        dwCryptTyp - Define-Konstanten von CRYPT_STRING_-Werten
                Rckgabe     : cReturn - String
        */
        LOCAL cReturn := ""        //Rckgabewert
        LOCAL dwLen := 0        //Lnge des Wertes
        LOCAL dwReturnLen := 0        //Lnge des Strings, das zurck gegeben wird
        LOCAL ptrReturn := ""         //Rckgabewert der WINAPI
        LOCAL lReturn := .f.
        local aTyp := { CRYPT_STRING_BASE64HEADER, CRYPT_STRING_BASE64, CRYPT_STRING_BINARY,;
                                CRYPT_STRING_HEX, CRYPT_STRING_BASE64_ANY, CRYPT_STRING_HEX_ANY, CRYPT_STRING_BASE64X509CRLHEADER,;
                                CRYPT_STRING_HEXADDR, CRYPT_STRING_HEXASCIIADDR, CRYPT_STRING_HEXRAW }


        if ! cString == ""

                //Nur Erlaubte Werte fr den Typ zulassen
                IF aScan( aTyp, dwCryptTyp ) > 0

                        //Daten vorbereiten

                        //Lnge ermitteln
                        dwLen := Len( cString )

                        //Lnge ermitteln, die der neue Stringwert haben wird. Dies ist fr den Umwandlungsaufruf ntig
                        //lReturn := IIF( CryptBinaryToStringA( @cString, dwLen, dwCryptTyp, NIL, @dwReturnLen )==1, .t., .f.)
                        lReturn := IIF( DllCall( "Crypt32.dll", DLL_STDCALL, "CryptBinaryToStringA", @cString, dwLen, dwCryptTyp, NIL, @dwReturnLen )==1, .t., .f.)
                        if lReturn
                                //Speicher fr den Ergebnis-String reservieren
                                ptrReturn := space(dwReturnLen)
                                //Wert codieren
                                //lReturn := CryptBinaryToStringA( @cString, dwLen, dwCryptTyp, @ptrReturn, @dwReturnLen )
                                lReturn := IIF( DllCall( "Crypt32.dll", DLL_STDCALL, "CryptBinaryToStringA", @cString, dwLen, dwCryptTyp, @ptrReturn, @dwReturnLen )==1, .t., .f.)
                                if lReturn
                                        //Daten wurden erfolgreich codiert
                                        //Wert aus dem Speicher holen
                                        cReturn := substr( ptrReturn, 1, dwReturnLen)
                                endif

                        ENDIF

                ENDIF

        ENDIF

RETURN cReturn


METHOD StringToBinary( cString , dwCryptTyp )
        /* Beschreibung : einen String des angegebenen Typs in einen normalen String umwandeln
                                                        Diese Methode ist eine Auslagerung des allgemenen in allen Methoden wiederholten Quellcodes
                Parameter    : cString - String
                                                        dwCryptTyp - Define-Konstanten von CRYPT_STRING_-Werten
                Rckgabe     : cReturn - String
        */
        LOCAL cReturn := ""        //Rckgabewert
        LOCAL dwLen := 0        //Lnge des Wertes
        LOCAL dwReturnLen := 0        //Lnge des Strings, das zurck gegeben wird
        LOCAL ptrReturn := ""         //Rckgabewert der WINAPI
        LOCAL lReturn := .f.
        local aTyp := { CRYPT_STRING_BASE64HEADER, CRYPT_STRING_BASE64, CRYPT_STRING_BINARY,;
                                CRYPT_STRING_HEX, CRYPT_STRING_BASE64_ANY, CRYPT_STRING_HEX_ANY, CRYPT_STRING_BASE64X509CRLHEADER,;
                                CRYPT_STRING_HEXADDR, CRYPT_STRING_HEXASCIIADDR, CRYPT_STRING_HEXRAW }


        if ! cString == ""

                //Nur Erlaubte Werte fr den Typ zulassen
                 IF aScan( aTyp, dwCryptTyp ) > 0

                        //Daten vorbereiten

                        //Lnge ermitteln
                        dwLen := Len( cString )

                        //Lnge ermitteln, die der neue Stringwert haben wird. Dies ist fr den Umwandlungsaufruf ntig
                        //lReturn := CryptStringToBinaryA( @cString, dwLen, dwCryptTyp, NIL, @dwReturnLen, NIL, NIL )
                        lReturn := IIF( DllCall( "Crypt32.dll", DLL_STDCALL, "CryptStringToBinaryA", @cString, dwLen, dwCryptTyp, NIL, @dwReturnLen, NIL, NIL )==1, .t., .f.)
                        if lReturn
                                //Speicher fr den Ergebnis-String reservieren
                                dwReturnLen++
                                ptrReturn := space(dwReturnLen)
                                //Wert decodieren
                                //lReturn := CryptStringToBinaryA( @cString, dwLen, dwCryptTyp, @ptrReturn, @dwReturnLen, NIL, NIL )
                                lReturn := IIF( DllCall( "Crypt32.dll", DLL_STDCALL, "CryptStringToBinaryA", @cString, dwLen, dwCryptTyp, @ptrReturn, @dwReturnLen, NIL, NIL )==1, .t., .f.)
                                if lReturn
                                        //Daten wurden erfolgreich decodiert
                                        //Wert aus dem Speicher holen
                                        cReturn := substr( ptrReturn, 1, dwReturnLen)
                                endif

                        endif

                ENDIF

        ENDIF


RETURN cReturn




DLLFUNCTION  CryptBinaryToStringA( @pbBinary, nBinary, nFlags, @cString, @pcchString )    USING STDCALL FROM CRYPT32.DLL
//DLLFUNCTION  CryptStringToBinaryA( @pszString, dwString, dwFlags, @pszBinary, @pcbBinary, @pdwSkip, @pdwFlags )    USING STDCALL FROM CRYPT32.DLL

//DLLFUNCTION  CryptBinaryToStringA( @pbBinary, nBinary, nFlags, @cString, @pcchString )    USING STDCALL FROM CRYPT32.DLL
//DLLFUNCTION  CryptStringToBinaryA( @pszString, dwString, dwFlags, @pszBinary, @pcbBinary, @pdwSkip, @pdwFlags )    USING STDCALL FROM CRYPT32.DLL